#!/usr/bin/env python
#------------------------------------------------------------------------------#
# Snapshot-Manager for ZevenOS-Neptune
# written by Leszek Lesner
# released under the terms of GPLv3
# 
# An application to install, create and remove snapshots from a live usb stick
#------------------------------------------------------------------------------#
import sys
import ConfigParser
import time
from PyQt4 import QtCore, QtGui

from snapshotmanager_ui import Ui_MainWindow
from os import popen, path, system
global config
config = ConfigParser.RawConfigParser()
global userconfig
userconfig = ConfigParser.RawConfigParser()
home = path.expanduser("~")
# There is a difference between snapshots made for the minimal and for the full edition
if path.exists("/etc/zeven_minimal"):
    version = "minimal"
elif path.exists("/etc/zeven_full"):
    version = "full"
else:
    version = "legacy"


class MyForm(QtGui.QMainWindow):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self.center()
        popen("touch /live/image/.foo")
        if not path.exists("/live/image/.foo"):
           QtGui.QMessageBox.information(None, "Warning", self.tr("Cannot write to live media. Are you using a live cd ? Snapshot Manager can only run on a live usb stick.")) 
        else:
           popen("rm /live/image/.foo")
        if not path.exists("/live/image/live/filesystem.module"):
            popen("echo 'filesystem.squashfs' > /live/image/live/filesystem.module")  
        if not path.exists("/live/image/snapshot-data/"):    # snapshot-data is used to store images & snapshots.cfg which contains metadata about snapshots
            popen("mkdir /live/image/snapshot-data/")
            #popen("cp /etc/snapshots.cfg /live/image/snapshot-data/snapshots.cfg") # create working copy which we can read & write without root privileges
            #popen("touch /live/image/snapshot-data/user-sn.cfg") # create working copy which we can read & write without root privileges
        #else:
        #    popen("diff /live/image/snapshot-data/snapshots.cfg /etc/snapshots.cfg | sed '/^[0-9][0-9]*/d; /^< /d; s/^. //; /^---$/d' > /tmp/snapshots.tmp ; mv /live/image/snapshot-data/snapshots.cfg /live/image/snapshot-data/snapshots.cfg.bak ; cat /live/image/snapshot-data/snapshots.cfg.bak /tmp/snapshots.tmp > /live/image/snapshot-data/snapshots.cfg")         
        self.loadsnapshotlist()
        self.ui.snap_combo.currentIndexChanged.connect(self.loadsnapshotdata) # Make the connections for buttons 
        self.ui.snap_install.clicked.connect(self.snap_install)
        self.ui.snap_remove.clicked.connect(self.snap_remove)
        self.ui.snap_create.clicked.connect(self.snap_create)
        self.ui.actionQuit.triggered.connect(self.quitbtnClicked)
        self.ui.actionManual_Install.triggered.connect(self.snap_manual_install)
        self.ui.actionAbout.triggered.connect(self.about_show)
        #self.ui.snap_remove.setEnabled(False)  # Handled now in load_snapshotdata
        self.ui.menuBar.hide()
        self.ui.menuBtn.setPopupMode(QtGui.QToolButton.MenuButtonPopup)
        self.ui.menuBtn.setMenu(self.ui.menuMenu)
        
    def quitbtnClicked(self):
        sys.exit(0)                 # Why is this here ? Too much copy & paste !? 

    def snap_create(self):          # Create the snapshot by calling the external snapshot bash script from persistent-creator package
        snap, ok = QtGui.QInputDialog.getText(self, 'Input Snapshotname', self.tr('Enter snapshot name for the new snapshot:'))
        snap = str(snap)
        if ok:
            if path.exists("/usr/bin/kdialog"):
                system("/usr/bin/snapshot")
            else:
                system("xterm -e /usr/bin/snapshot")
            popen("mkdir -p /live/image/snapshot-data")
            popen("mv /live/image/live-sn.squashfs /live/image/snapshot-data/"+snap+"-sn.squashfs")
            popen("touch /live/image/snapshot-data/" + snap + ".png")           # Create a null png file because we are searching for it
            size = popen("ls -lh /live/image/snapshot-data/"+snap+"-sn.squashfs | cut -d ' ' -f 5").readline()  # Get the file size of the new image
            userconfig.add_section(snap)
            userconfig.set(snap, 'desc', 'Custom Snapshot (~ '+size.strip()+'B) ') 
            userconfig.set(snap, 'img', 'None')
            userconfig.set(snap, 'url', 'None')
            global version
            userconfig.set(snap, 'version', version)
            with open('/live/image/snapshot-data/user-sn.cfg', 'wb') as configfile:
                userconfig.write(configfile)
            self.loadsnapshotlist()
            self.snap_install(snap)
            if path.exists("/live/image/live/"+snap+"-sn.squashfs"):
                self.ui.snap_remove.setEnabled(True)
                self.ui.snap_install.setEnabled(False)
            else:
                self.ui.snap_remove.setEnabled(False)
                self.ui.snap_install.setEnabled(False)

    def snap_manual_install(self):          # Manual install for files downloaded somewhere from the net or got from a friend
        snaplocation = QtGui.QFileDialog.getOpenFileName(None,self.tr("Open Snapshot"), home) 
        if snaplocation:
            snap, ok = QtGui.QInputDialog.getText(self, 'Input Snapshotname', self.tr('Enter snapshot name for the new snapshot:'))
            snap = str(snap)
            if ok:
                popen("mkdir -p /live/image/snapshot-data")
                popen("cp " + str(snaplocation) + " /live/image/snapshot-data/"+snap+"-sn.squashfs")
                popen("touch /live/image/snapshot-data/" + snap + ".png")           # Create a null png file because we are searching for it
                size = popen("ls -lh /live/image/snapshot-data/"+snap+"-sn.squashfs | cut -d ' ' -f 5").readline()  # Get the file size of the new image
                userconfig.add_section(snap)
                userconfig.set(snap, 'desc', 'Custom Snapshot (~ '+size.strip()+'B) ')
                userconfig.set(snap, 'img', 'None')
                userconfig.set(snap, 'url', 'None')
                global version
                userconfig.set(snap, 'version', version)
                with open('/live/image/snapshot-data/user-sn.cfg', 'wb') as configfile:
                    userconfig.write(configfile)
                self.loadsnapshotlist()
                self.snap_install(snap)
                if path.exists("/live/image/live/"+snap+"-sn.squashfs"):
                    self.ui.snap_remove.setEnabled(True)
                    self.ui.snap_install.setEnabled(False)
                else:
                    self.ui.snap_remove.setEnabled(False)
                    self.ui.snap_install.setEnabled(False)
            
    def snap_remove(self):
        snap = str(self.ui.snap_combo.currentText()) 
        url = str(self.ui.snap_url.text())
        if path.exists("/live/image/live/"+snap+"-sn.squashfs"):  # Only remove something if its there 
            popen("rm /live/image/live/"+snap+"-sn.squashfs /live/image/live/"+snap+"-sn.module")
            popen("rm /live/image/snapshot-data/"+snap+"-syslinux.backup")
            popen("patch -R /live/image/syslinux.cfg < /live/image/snapshot-data/"+snap+"-syslinux.patch")
            if url == 'None':          # Only for manually created snapshots
                userconfig.remove_section(snap)
                with open('/live/image/snapshot-data/user-sn.cfg', 'wb') as configfile:
                    userconfig.write(configfile)
                self.loadsnapshotlist()
            QtGui.QMessageBox.information(None, "Information", self.tr("Removal complete!")) 
            self.ui.snap_remove.setEnabled(False)
            self.ui.snap_desc.setText("")
            self.ui.snap_url.setText("")
            try:
                self.ui.snap_img.setPixmap(None)
            except:
                print("Setting pixmaps to None failed")
            

    def snap_install(self):
        snap = str(self.ui.snap_combo.currentText())
        url = str(self.ui.snap_url.text())
        pbar = QtGui.QProgressBar()
        pbar.setValue(25)
        self.ui.verticalLayout.addWidget(pbar)
        popen("mkdir -p /live/image/snapshot-data/")
        if not path.exists("/live/image/snapshot-data/"+snap+"-sn.squashfs"):    # Download the snapshot image (Damn I cannot program progressbars here so only a xterm here)
            popen("xterm -e 'wget -c " + url + " -O /live/image/snapshot-data/"+snap+"-sn.squashfs.unfinished ; mv /live/image/snapshot-data/"+snap+"-sn.squashfs.unfinished  /live/image/snapshot-data/"+snap+"-sn.squashfs'")
        pbar.setValue(50)
        popen("mv /live/image/snapshot-data/"+snap+"-sn.squashfs /live/image/live/"+snap+"-sn.squashfs")
        popen("echo 'filesystem.squashfs' > /live/image/live/"+snap+"-sn.module")   # Create the .module file that we need for the separate menu entry
        popen("echo '"+snap+"-sn.squashfs' >> /live/image/live/"+snap+"-sn.module")
        popen("cp /live/image/syslinux.cfg /live/image/snapshot-data/"+snap+"-syslinux.backup")
        pbar.setValue(75)
        popen("echo ' ' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'label ubnentry10' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")  # Create the new menu entry
        popen("echo 'menu label Neptune Live ("+snap+" Snapshot)' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'kernel /live/vmlinuz2' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'append initrd=/live/initrd2.img boot=live union=aufs locale=de_DE.UTF-8 keyb=de timezone=Europe/Berlin klayout=de kvariant=nodeadkeys swapon splash noprompt quiet init=/lib/systemd/systemd module="+snap+"-sn' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo ' ' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'label ubnentry10' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'menu label Neptune Live ("+snap+" Snapshot EN)' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'kernel /live/vmlinuz2' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'append initrd=/live/initrd2.img boot=live union=aufs locale=en_US.UTF-8 keyb=en swapon splash noprompt quiet init=/lib/systemd/systemd module="+snap+"-sn' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("diff -u /live/image/syslinux.cfg /live/image/snapshot-data/"+snap+"-syslinux.backup > /live/image/snapshot-data/"+snap+"-syslinux.patch")
        popen("patch /live/image/syslinux.cfg < /live/image/snapshot-data/"+snap+"-syslinux.patch")
        pbar.setValue(100)
        QtGui.QMessageBox.information(None, "Information", self.tr("Installation complete. Please reboot to use the installed snapshot")) 
        pbar.hide()
        self.ui.snap_remove.setEnabled(True)
        self.ui.snap_install.setEnabled(False)


    def snap_install(self, snap):
        if snap == False:
             snap = str(self.ui.snap_combo.currentText())
             url = str(self.ui.snap_url.text())
        pbar = QtGui.QProgressBar()
        pbar.setValue(25)
        self.ui.verticalLayout.addWidget(pbar)
        popen("mkdir -p /live/image/snapshot-data/")
        if not path.exists("/live/image/snapshot-data/"+snap+"-sn.squashfs"):    # Download the snapshot image (Damn I cannot program progressbars here so only a xterm here)
            popen("xterm -e 'wget -c " + url + " -O /live/image/snapshot-data/"+snap+"-sn.squashfs.unfinished ; mv /live/image/snapshot-data/"+snap+"-sn.squashfs.unfinished  /live/image/snapshot-data/"+snap+"-sn.squashfs'")
        pbar.setValue(50)
        popen("mv /live/image/snapshot-data/"+snap+"-sn.squashfs /live/image/live/"+snap+"-sn.squashfs")
        popen("echo 'filesystem.squashfs' > /live/image/live/"+snap+"-sn.module")   # Create the .module file that we need for the separate menu entry
        popen("echo '"+snap+"-sn.squashfs' >> /live/image/live/"+snap+"-sn.module")
        popen("cp /live/image/syslinux.cfg /live/image/snapshot-data/"+snap+"-syslinux.backup")
        pbar.setValue(75)
        popen("echo ' ' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'label ubnentry10' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")  # Create the new menu entry
        popen("echo 'menu label Neptune Live ("+snap+" Snapshot)' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'kernel /live/vmlinuz2' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'append initrd=/live/initrd2.img boot=live union=aufs locale=de_DE.UTF-8 keyb=de timezone=Europe/Berlin klayout=de kvariant=nodeadkeys swapon splash noprompt quiet module="+snap+"-sn' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo ' ' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'label ubnentry10' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'menu label Neptune Live ("+snap+" Snapshot EN)' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'kernel /live/vmlinuz2' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("echo 'append initrd=/live/initrd2.img boot=live union=aufs locale=en_US.UTF-8 keyb=en swapon splash noprompt quiet module="+snap+"-sn' >> /live/image/snapshot-data/"+snap+"-syslinux.backup")
        popen("diff -u /live/image/syslinux.cfg /live/image/snapshot-data/"+snap+"-syslinux.backup > /live/image/snapshot-data/"+snap+"-syslinux.patch")
        popen("patch /live/image/syslinux.cfg < /live/image/snapshot-data/"+snap+"-syslinux.patch")
        pbar.setValue(100)
        QtGui.QMessageBox.information(None, "Information", self.tr("Installation complete. Please reboot to use the installed snapshot")) 
        pbar.hide()
        self.ui.snap_remove.setEnabled(True)
        self.ui.snap_install.setEnabled(False)

    def loadsnapshotdata(self,snap):    # Load the snapshot metadata
        snap = str(self.ui.snap_combo.currentText())
        if not snap == '':
            try:
                self.ui.snap_desc.setText(config.get(snap, "desc"))
                snap_image_location = config.get(snap, "img")
            except:
                self.ui.snap_desc.setText(userconfig.get(snap, "desc"))
                snap_image_location = userconfig.get(snap, "img")
            if not path.exists("/live/image/snapshot-data/" + snap + ".png"):
                popen("wget " + snap_image_location + " -O /live/image/snapshot-data/"+ snap + ".png")
            snap_image_location = "/live/image/snapshot-data/"+ snap + ".png"
            snap_image = QtGui.QPixmap()
            snap_image.load(snap_image_location, format = None, flags = QtCore.Qt.AutoColor)
            self.ui.snap_img.setPixmap(snap_image)
            self.ui.snap_img.setScaledContents(True)
            try:
                self.ui.snap_url.setText(config.get(snap, "url"))
            except:
                self.ui.snap_url.setText(userconfig.get(snap, "url"))
            if path.exists("/live/image/live/"+snap+"-sn.squashfs"):
                print "Enable remove button"
                self.ui.snap_remove.setEnabled(True)
                self.ui.snap_install.setEnabled(False)
            else:
                print "Disable remove button"
                self.ui.snap_remove.setEnabled(False)
                if self.ui.snap_combo.count() > 0:
                    self.ui.snap_install.setEnabled(True)
                else:
                    self.ui.snap_install.setEnabled(False)
        else:
            self.ui.snap_install.setEnabled(False)
            self.ui.snap_remove.setEnabled(False)
        

    def loadsnapshotlist(self):
        """ Load Snapshot global config & user config """
        global version
        #print "Version = " + version
        self.ui.snap_combo.clear()
        config.read('/etc/snapshots.cfg')
        for i in config.sections():
            try:
                if config.get(i, "version") == version:
                    self.ui.snap_combo.addItem(i, userData=None) 
            except:
                print "Snapshot without version information found: " + i
        self.loadsnapshotdata(None)
        if path.exists("/live/image/snapshot-data/user-sn.cfg"):
            userconfig.read('/live/image/snapshot-data/user-sn.cfg')
            for i in userconfig.sections():
                try:
                    if userconfig.get(i, "version") == version:
                        self.ui.snap_combo.addItem(i, userData=None) 
                except:
                    print "Snapshot without version information found: " + i
            self.loadsnapshotdata(None)
        
    def about_show(self):
        QtGui.QMessageBox.information(None, "About", "Snapshot Manager Version 1.0.2\nby Leszek Lesner\nreleased under the terms of GPLv3")
        
    def center(self):
        screen = QtGui.QDesktopWidget().screenGeometry()
        size =  self.geometry()
        self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)

        

if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    glob_trans=QtCore.QTranslator()
    glob_trans.load("/usr/share/qt4/translations/qt_"+unicode(QtCore.QLocale.system().name())[:2]+".qm")
    app.installTranslator(glob_trans)
    print ("trying to load localization file /usr/share/qt4/translations/qt_"+unicode(QtCore.QLocale.system().name())[:2]+".qm") # Debug only
    translator=QtCore.QTranslator()
    directory="/usr/share/snapshot-manager"
    translator.load(directory+"/snapshot-manager_ui-"+unicode(QtCore.QLocale.system().name()[:2])+u".qm")
    print ("trying to load localization file "+ directory+"/snapshot-manager_ui-"+unicode(QtCore.QLocale.system().name())[:2]+".qm") # Debug only
    app.installTranslator(translator)
    myapp = MyForm()
    myapp.show()
    sys.exit(app.exec_())
